@@ -160,8 +160,25 @@ fn impl_def_from_trait(
     if trait_items.is_empty() {
         return None;
     }
-    let impl_def =
-        make::impl_trait(trait_path.clone(), make::ext::ident_path(&annotated_name.text()));
+    let impl_def = {
+        use syntax::ast::Impl;
+        let text = generate_trait_impl_text(adt, trait_path.to_string().as_str(), "");
+        let parse = syntax::SourceFile::parse(&text);
+        let node = match parse.tree().syntax().descendants().find_map(Impl::cast) {
+            Some(it) => it,
+            None => {
+                panic!(
+                    "Failed to make ast node `{}` from text {}",
+                    std::any::type_name::<Impl>(),
+                    text
+                )
+            }
+        };
+        let node = node.clone_subtree();
+        assert_eq!(node.syntax().text_range().start(), 0.into());
+        node
+    };
+
     let (impl_def, first_assoc_item) =
         add_trait_assoc_items_to_impl(sema, trait_items, trait_, impl_def, target_scope);
 
@@ -693,7 +710,7 @@ struct Foo {
 
 impl PartialOrd for Foo {
     $0fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
-        self.bin.partial_cmp(other.bin)
+        self.bin.partial_cmp(&other.bin)
     }
 }
 "#,
@@ -722,7 +739,15 @@ struct Foo {
 
 impl PartialOrd for Foo {
     $0fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
-        (self.bin, self.bar, self.baz).partial_cmp((other.bin, other.bar, other.baz))
+        match self.bin.partial_cmp(&other.bin) {
+            Some(core::cmp::Ordering::Equal) => {}
+            ord => return ord,
+        }
+        match self.bar.partial_cmp(&other.bar) {
+            Some(core::cmp::Ordering::Equal) => {}
+            ord => return ord,
+        }
+        self.baz.partial_cmp(&other.baz)
     }
 }
 "#,
@@ -743,120 +768,15 @@ struct Foo(usize, usize, usize);
 
 impl PartialOrd for Foo {
     $0fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
-        (self.0, self.1, self.2).partial_cmp((other.0, other.1, other.2))
-    }
-}
-"#,
-        )
-    }
-
-    #[test]
-    fn add_custom_impl_partial_ord_enum() {
-        check_assist(
-            replace_derive_with_manual_impl,
-            r#"
-//- minicore: ord
-#[derive(Partial$0Ord)]
-enum Foo {
-    Bin,
-    Bar,
-    Baz,
-}
-"#,
-            r#"
-enum Foo {
-    Bin,
-    Bar,
-    Baz,
-}
-
-impl PartialOrd for Foo {
-    $0fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
-        core::mem::discriminant(self).partial_cmp(core::mem::discriminant(other))
-    }
-}
-"#,
-        )
-    }
-
-    #[test]
-    fn add_custom_impl_partial_ord_record_enum() {
-        check_assist(
-            replace_derive_with_manual_impl,
-            r#"
-//- minicore: ord
-#[derive(Partial$0Ord)]
-enum Foo {
-    Bar {
-        bin: String,
-    },
-    Baz {
-        qux: String,
-        fez: String,
-    },
-    Qux {},
-    Bin,
-}
-"#,
-            r#"
-enum Foo {
-    Bar {
-        bin: String,
-    },
-    Baz {
-        qux: String,
-        fez: String,
-    },
-    Qux {},
-    Bin,
-}
-
-impl PartialOrd for Foo {
-    $0fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
-        match (self, other) {
-            (Self::Bar { bin: l_bin }, Self::Bar { bin: r_bin }) => l_bin.partial_cmp(r_bin),
-            (Self::Baz { qux: l_qux, fez: l_fez }, Self::Baz { qux: r_qux, fez: r_fez }) => {
-                (l_qux, l_fez).partial_cmp((r_qux, r_fez))
-            }
-            _ => core::mem::discriminant(self).partial_cmp(core::mem::discriminant(other)),
+        match self.0.partial_cmp(&other.0) {
+            Some(core::cmp::Ordering::Equal) => {}
+            ord => return ord,
         }
-    }
-}
-"#,
-        )
-    }
-
-    #[test]
-    fn add_custom_impl_partial_ord_tuple_enum() {
-        check_assist(
-            replace_derive_with_manual_impl,
-            r#"
-//- minicore: ord
-#[derive(Partial$0Ord)]
-enum Foo {
-    Bar(String),
-    Baz(String, String),
-    Qux(),
-    Bin,
-}
-"#,
-            r#"
-enum Foo {
-    Bar(String),
-    Baz(String, String),
-    Qux(),
-    Bin,
-}
-
-impl PartialOrd for Foo {
-    $0fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
-        match (self, other) {
-            (Self::Bar(l0), Self::Bar(r0)) => l0.partial_cmp(r0),
-            (Self::Baz(l0, l1), Self::Baz(r0, r1)) => {
-                (l0, l1).partial_cmp((r0, r1))
-            }
-            _ => core::mem::discriminant(self).partial_cmp(core::mem::discriminant(other)),
+        match self.1.partial_cmp(&other.1) {
+            Some(core::cmp::Ordering::Equal) => {}
+            ord => return ord,
         }
+        self.2.partial_cmp(&other.2)
     }
 }
 "#,
@@ -1145,6 +1065,54 @@ impl Debug for Foo {
         )
     }
 
+    #[test]
+    fn add_custom_impl_default_generic_record_struct() {
+        check_assist(
+            replace_derive_with_manual_impl,
+            r#"
+//- minicore: default
+#[derive(Defau$0lt)]
+struct Foo<T, U> {
+    foo: T,
+    bar: U,
+}
+"#,
+            r#"
+struct Foo<T, U> {
+    foo: T,
+    bar: U,
+}
+
+impl<T, U> Default for Foo<T, U> {
+    $0fn default() -> Self {
+        Self { foo: Default::default(), bar: Default::default() }
+    }
+}
+"#,
+        )
+    }
+
+    #[test]
+    fn add_custom_impl_clone_generic_tuple_struct_with_bounds() {
+        check_assist(
+            replace_derive_with_manual_impl,
+            r#"
+//- minicore: clone
+#[derive(Clo$0ne)]
+struct Foo<T: Clone>(T, usize);
+"#,
+            r#"
+struct Foo<T: Clone>(T, usize);
+
+impl<T: Clone> Clone for Foo<T> {
+    $0fn clone(&self) -> Self {
+        Self(self.0.clone(), self.1.clone())
+    }
+}
+"#,
+        )
+    }
+
     #[test]
     fn test_ignore_derive_macro_without_input() {
         check_assist_not_applicable(
